@@ -37,6 +37,7 @@
 #include <ctype.h>
 
 #include "clamav.h"
+#include "str.h"
 #include "others.h"
 #include "unarj.h"
 #include "textnorm.h"
@@ -160,7 +161,7 @@ typedef struct arj_decode_tag {
     int status;
 } arj_decode_t;
 
-static int fill_buf(arj_decode_t *decode_data, int n)
+static cl_error_t fill_buf(arj_decode_t *decode_data, int n)
 {
     if (decode_data->status == CL_EFORMAT)
         return CL_EFORMAT;
@@ -194,7 +195,7 @@ static int fill_buf(arj_decode_t *decode_data, int n)
     return CL_SUCCESS;
 }
 
-static int init_getbits(arj_decode_t *decode_data)
+static cl_error_t init_getbits(arj_decode_t *decode_data)
 {
     decode_data->bit_buf     = 0;
     decode_data->sub_bit_buf = 0;
@@ -211,7 +212,7 @@ static unsigned short arj_getbits(arj_decode_t *decode_data, int n)
     return x;
 }
 
-static int decode_start(arj_decode_t *decode_data)
+static cl_error_t decode_start(arj_decode_t *decode_data)
 {
     decode_data->blocksize = 0;
     return init_getbits(decode_data);
@@ -228,8 +229,8 @@ static cl_error_t write_text(int ofd, unsigned char *data, size_t length)
     return CL_SUCCESS;
 }
 
-static int make_table(arj_decode_t *decode_data, int nchar, unsigned char *bitlen, int tablebits,
-                      unsigned short *table, int tablesize)
+static cl_error_t make_table(arj_decode_t *decode_data, int nchar, unsigned char *bitlen, int tablebits,
+                             unsigned short *table, int tablesize)
 {
     unsigned short count[17], weight[17], start[18], *p;
     unsigned int i, k, len, ch, jutbits, avail, nextcode, mask;
@@ -337,7 +338,7 @@ static int make_table(arj_decode_t *decode_data, int nchar, unsigned char *bitle
     return CL_SUCCESS;
 }
 
-static int read_pt_len(arj_decode_t *decode_data, int nn, int nbit, int i_special)
+static cl_error_t read_pt_len(arj_decode_t *decode_data, int nn, int nbit, int i_special)
 {
     int i, n;
     short c;
@@ -393,7 +394,7 @@ static int read_pt_len(arj_decode_t *decode_data, int nn, int nbit, int i_specia
     return CL_SUCCESS;
 }
 
-static int read_c_len(arj_decode_t *decode_data)
+static cl_error_t read_c_len(arj_decode_t *decode_data)
 {
     short i, c, n;
     unsigned short mask;
@@ -541,9 +542,9 @@ static uint16_t decode_p(arj_decode_t *decode_data)
     return j;
 }
 
-static int decode(arj_metadata_t *metadata)
+static cl_error_t decode(arj_metadata_t *metadata)
 {
-    int ret;
+    cl_error_t ret;
 
     arj_decode_t decode_data;
     uint32_t count = 0, out_ptr = 0;
@@ -695,9 +696,9 @@ static uint16_t decode_len(arj_decode_t *decode_data)
     return c;
 }
 
-static int decode_f(arj_metadata_t *metadata)
+static cl_error_t decode_f(arj_metadata_t *metadata)
 {
-    int ret;
+    cl_error_t ret;
 
     arj_decode_t decode_data, *dd;
     uint32_t count = 0, out_ptr = 0;
@@ -786,7 +787,7 @@ static int decode_f(arj_metadata_t *metadata)
     return CL_SUCCESS;
 }
 
-static int arj_unstore(arj_metadata_t *metadata, int ofd, uint32_t len)
+static cl_error_t arj_unstore(arj_metadata_t *metadata, int ofd, uint32_t len)
 {
     const unsigned char *data;
     uint32_t rem;
@@ -893,7 +894,7 @@ static int arj_read_main_header(arj_metadata_t *metadata)
 
     filename_max_len = (header_size + sizeof(header_size)) - (metadata->offset - orig_offset);
     if (filename_max_len > header_size) {
-        cli_dbgmsg("UNARJ: Format error. First Header Size invalid");
+        cli_dbgmsg("UNARJ: Format error. First Header Size invalid\n");
         ret = FALSE;
         goto done;
     }
@@ -911,7 +912,7 @@ static int arj_read_main_header(arj_metadata_t *metadata)
 
     comment_max_len = (header_size + sizeof(header_size)) - (metadata->offset - orig_offset);
     if (comment_max_len > header_size) {
-        cli_dbgmsg("UNARJ: Format error. First Header Size invalid");
+        cli_dbgmsg("UNARJ: Format error. First Header Size invalid\n");
         ret = FALSE;
         goto done;
     }
@@ -968,7 +969,7 @@ done:
     return ret;
 }
 
-static int arj_read_file_header(arj_metadata_t *metadata)
+static cl_error_t arj_read_file_header(arj_metadata_t *metadata)
 {
     uint16_t header_size, count;
     const char *filename, *comment;
@@ -976,7 +977,7 @@ static int arj_read_file_header(arj_metadata_t *metadata)
     struct text_norm_state fnstate, comstate;
     unsigned char *fnnorm  = NULL;
     unsigned char *comnorm = NULL;
-    uint32_t ret           = CL_SUCCESS;
+    cl_error_t ret         = CL_SUCCESS;
 
     size_t filename_max_len = 0;
     size_t filename_len     = 0;
@@ -1002,7 +1003,7 @@ static int arj_read_file_header(arj_metadata_t *metadata)
     }
     if ((header_size + sizeof(header_size)) > (metadata->map->real_len - metadata->offset)) {
         cli_dbgmsg("arj_read_file_header: invalid header_size: %u, exceeds length of file.\n", header_size);
-        ret = FALSE;
+        ret = CL_EFORMAT;
         goto done;
     }
     if (fmap_readn(metadata->map, &file_hdr, metadata->offset, 30) != 30) {
@@ -1038,16 +1039,21 @@ static int arj_read_file_header(arj_metadata_t *metadata)
 
     filename_max_len = (header_size + sizeof(header_size)) - (metadata->offset - orig_offset);
     if (filename_max_len > header_size) {
-        cli_dbgmsg("UNARJ: Format error. First Header Size invalid");
-        ret = FALSE;
+        cli_dbgmsg("UNARJ: Format error. First Header Size invalid\n");
+        ret = CL_EFORMAT;
         goto done;
     }
     if (filename_max_len > 0) {
-        fnnorm   = cli_calloc(sizeof(unsigned char), filename_max_len + 1);
-        filename = fmap_need_offstr(metadata->map, metadata->offset, filename_max_len + 1);
-        if (!filename || !fnnorm) {
+        fnnorm = cli_calloc(sizeof(unsigned char), filename_max_len + 1);
+        if (!fnnorm) {
             cli_dbgmsg("UNARJ: Unable to allocate memory for filename\n");
-            ret = FALSE;
+            ret = CL_EMEM;
+            goto done;
+        }
+        filename = fmap_need_offstr(metadata->map, metadata->offset, filename_max_len + 1);
+        if (!filename) {
+            cli_dbgmsg("UNARJ: Filename is out of file\n");
+            ret = CL_EFORMAT;
             goto done;
         }
         filename_len = CLI_STRNLEN(filename, filename_max_len);
@@ -1056,16 +1062,21 @@ static int arj_read_file_header(arj_metadata_t *metadata)
 
     comment_max_len = (header_size + sizeof(header_size)) - (metadata->offset - orig_offset);
     if (comment_max_len > header_size) {
-        cli_dbgmsg("UNARJ: Format error. First Header Size invalid");
-        ret = FALSE;
+        cli_dbgmsg("UNARJ: Format error. First Header Size invalid\n");
+        ret = CL_EFORMAT;
         goto done;
     }
     if (comment_max_len > 0) {
         comnorm = cli_calloc(sizeof(unsigned char), comment_max_len + 1);
-        comment = fmap_need_offstr(metadata->map, metadata->offset, comment_max_len + 1);
-        if (!comment || !comnorm) {
+        if (!comnorm) {
             cli_dbgmsg("UNARJ: Unable to allocate memory for comment\n");
-            ret = FALSE;
+            ret = CL_EMEM;
+            goto done;
+        }
+        comment = fmap_need_offstr(metadata->map, metadata->offset, comment_max_len + 1);
+        if (!comment) {
+            cli_dbgmsg("UNARJ: comment is out of file\n");
+            ret = CL_EFORMAT;
             goto done;
         }
         comment_len += CLI_STRNLEN(comment, comment_max_len);
@@ -1128,7 +1139,7 @@ done:
     return ret;
 }
 
-int cli_unarj_open(fmap_t *map, const char *dirname, arj_metadata_t *metadata, size_t off)
+cl_error_t cli_unarj_open(fmap_t *map, const char *dirname, arj_metadata_t *metadata, size_t off)
 {
     UNUSEDPARAM(dirname);
     cli_dbgmsg("in cli_unarj_open\n");
@@ -1145,7 +1156,7 @@ int cli_unarj_open(fmap_t *map, const char *dirname, arj_metadata_t *metadata, s
     return CL_SUCCESS;
 }
 
-int cli_unarj_prepare_file(const char *dirname, arj_metadata_t *metadata)
+cl_error_t cli_unarj_prepare_file(const char *dirname, arj_metadata_t *metadata)
 {
     cli_dbgmsg("in cli_unarj_prepare_file\n");
     if (!metadata || !dirname) {
@@ -1159,9 +1170,9 @@ int cli_unarj_prepare_file(const char *dirname, arj_metadata_t *metadata)
     return arj_read_file_header(metadata);
 }
 
-int cli_unarj_extract_file(const char *dirname, arj_metadata_t *metadata)
+cl_error_t cli_unarj_extract_file(const char *dirname, arj_metadata_t *metadata)
 {
-    int ret = CL_SUCCESS;
+    cl_error_t ret = CL_SUCCESS;
     char filename[1024];
 
     cli_dbgmsg("in cli_unarj_extract_file\n");
